@@ -9,7 +9,6 @@ use base64::encode;
 use itertools::Itertools;
 use log::warn;
 use maplit::hashmap;
-use serde::{Deserialize, Serialize};
 use serde_json::{json, Value};
 
 use crate::{DifferenceType, PactSpecification};
@@ -22,7 +21,7 @@ use crate::query_strings::{query_from_json, query_to_json, v3_query_from_json};
 use crate::v4::http_parts::HttpRequest;
 
 /// Struct that defines the request.
-#[derive(Serialize, Deserialize, Debug, Clone, Eq)]
+#[derive(Debug, Clone, Eq)]
 pub struct Request {
   /// Request method
   pub method: String,
@@ -56,14 +55,26 @@ impl HttpPart for Request {
     &self.body
   }
 
+  fn body_mut(&mut self) -> &mut OptionalBody {
+    &mut self.body
+  }
+
   fn matching_rules(&self) -> &MatchingRules {
     &self.matching_rules
   }
 
+  fn matching_rules_mut(&mut self) -> &mut MatchingRules {
+    &mut self.matching_rules
+  }
+
   fn generators(&self) -> &Generators {
     &self.generators
   }
 
+  fn generators_mut(&mut self) -> &mut Generators {
+    &mut self.generators
+  }
+
   fn lookup_content_type(&self) -> Option<String> {
     self.lookup_header_value(&"content-type".to_string())
   }
@@ -178,7 +189,7 @@ impl Request {
         map.insert("headers".to_string(), headers_to_json(&self.headers.clone().unwrap()));
       }
       match self.body {
-        OptionalBody::Present(ref body, _) => if self.content_type().unwrap_or_default().is_json() {
+        OptionalBody::Present(ref body, _, _) => if self.content_type().unwrap_or_default().is_json() {
           match serde_json::from_slice(body) {
             Ok(json_body) => { map.insert("body".to_string(), json_body); },
             Err(err) => {
@@ -263,6 +274,7 @@ mod tests {
 
   use expectest::prelude::*;
   use maplit::hashmap;
+  use serde_json::Value;
 
   use crate::bodies::OptionalBody;
   use crate::content_types::{HTML, JSON, XML};
@@ -321,31 +333,31 @@ mod tests {
         headers: Some(hashmap!{ "CONTENT-TYPE".to_string() => vec!["application/json; charset=UTF-8".to_string()] }), .. request.clone() }.content_type().unwrap_or_default().to_string())
       .to(be_equal_to("application/json;charset=utf-8"));
     expect!(Request {
-        body: OptionalBody::Present("{\"json\": true}".into(), None), .. request.clone() }.content_type().unwrap_or_default().to_string())
+        body: OptionalBody::Present("{\"json\": true}".into(), None, None), .. request.clone() }.content_type().unwrap_or_default().to_string())
       .to(be_equal_to("application/json"));
     expect!(Request {
-        body: OptionalBody::Present("{}".into(), None), .. request.clone() }.content_type().unwrap_or_default().to_string())
+        body: OptionalBody::Present("{}".into(), None, None), .. request.clone() }.content_type().unwrap_or_default().to_string())
       .to(be_equal_to("application/json"));
     expect!(Request {
-        body: OptionalBody::Present("[]".into(), None), .. request.clone() }.content_type().unwrap_or_default().to_string())
+        body: OptionalBody::Present("[]".into(), None, None), .. request.clone() }.content_type().unwrap_or_default().to_string())
       .to(be_equal_to("application/json"));
     expect!(Request {
-        body: OptionalBody::Present("[1,2,3]".into(), None), .. request.clone() }.content_type().unwrap_or_default().to_string())
+        body: OptionalBody::Present("[1,2,3]".into(), None, None), .. request.clone() }.content_type().unwrap_or_default().to_string())
       .to(be_equal_to("application/json"));
     expect!(Request {
-        body: OptionalBody::Present("\"string\"".into(), None), .. request.clone() }.content_type().unwrap_or_default().to_string())
+        body: OptionalBody::Present("\"string\"".into(), None, None), .. request.clone() }.content_type().unwrap_or_default().to_string())
       .to(be_equal_to("application/json"));
     expect!(Request {
-        body: OptionalBody::Present("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<json>false</json>".into(), None), .. request.clone() }.content_type().unwrap_or_default().to_string())
+        body: OptionalBody::Present("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<json>false</json>".into(), None, None), .. request.clone() }.content_type().unwrap_or_default().to_string())
       .to(be_equal_to("application/xml"));
     expect!(Request {
-        body: OptionalBody::Present("<json>false</json>".into(), None), .. request.clone() }.content_type().unwrap_or_default().to_string())
+        body: OptionalBody::Present("<json>false</json>".into(), None, None), .. request.clone() }.content_type().unwrap_or_default().to_string())
       .to(be_equal_to("application/xml"));
     expect!(Request {
-        body: OptionalBody::Present("this is not json".into(), None), .. request.clone() }.content_type().unwrap_or_default().to_string())
+        body: OptionalBody::Present("this is not json".into(), None, None), .. request.clone() }.content_type().unwrap_or_default().to_string())
       .to(be_equal_to("text/plain"));
     expect!(Request {
-        body: OptionalBody::Present("<html><body>this is also not json</body></html>".into(), None), .. request.clone() }.content_type().unwrap_or_default().to_string())
+        body: OptionalBody::Present("<html><body>this is also not json</body></html>".into(), None, None), .. request.clone() }.content_type().unwrap_or_default().to_string())
       .to(be_equal_to("text/html"));
   }
 
@@ -469,7 +481,7 @@ mod tests {
   fn request_to_json_with_json_body() {
     let request = Request { headers: Some(hashmap!{
         "Content-Type".to_string() => vec!["application/json".to_string()]
-    }), body: OptionalBody::Present(r#"{"key": "value"}"#.into(), None), .. Request::default() };
+    }), body: OptionalBody::Present(r#"{"key": "value"}"#.into(), None, None), .. Request::default() };
     expect!(request.to_json(&PactSpecification::V3).to_string()).to(
       be_equal_to(r#"{"body":{"key":"value"},"headers":{"Content-Type":"application/json"},"method":"GET","path":"/"}"#)
     );
@@ -479,7 +491,7 @@ mod tests {
   #[test]
   fn request_to_json_with_non_json_body() {
     let request = Request { headers: Some(hashmap!{ "Content-Type".to_string() => vec!["text/plain".to_string()] }),
-      body: OptionalBody::Present("This is some text".into(), None), .. Request::default() };
+      body: OptionalBody::Present("This is some text".into(), None, None), .. Request::default() };
     expect!(request.to_json(&PactSpecification::V3).to_string()).to(
       be_equal_to(r#"{"body":"This is some text","headers":{"Content-Type":"text/plain"},"method":"GET","path":"/"}"#)
     );
@@ -541,13 +553,14 @@ mod tests {
       generators: Default::default(),
     };
 
-    let json = serde_json::to_string(&original_request).expect("could not serialize");
+    let json = serde_json::to_string(&original_request
+      .to_json(&PactSpecification::V3)).expect("could not serialize");
 
-    let serialized_and_deserialized_request =
+    let serialized_and_deserialized_request: Value =
       serde_json::from_str(&json).expect("could not deserialize");
 
     expect!(original_request
-        .differences_from(&serialized_and_deserialized_request)
+        .differences_from(&Request::from_json(&serialized_and_deserialized_request, &PactSpecification::V3).unwrap())
         .iter())
       .to(be_empty());
   }
